import {Recipe} from '../common/recipe';
import {endpoint, HttpResponse, operation, validation as v} from '../common';
import {backend} from '../backend';

interface AddResourceRecipeParams {
	endpointName: string;
}

export const addResourceRecipe = (params: AddResourceRecipeParams): Recipe => (a) => {
	const operations = {
		fetch: operation({
			name: 'fetch' as const,
		}),
	};

	const theEndpoint = endpoint({
		name: params.endpointName,
		schema: v.object({
			username: v.string(),
		}),
	})
		.param('resourceId')
		.can('fetch');

	const enhancedApp = a.app
		.operation(operations.fetch)
		.endpoint(theEndpoint);

	const theBackend = a.backend ?? backend({
		app: enhancedApp,
	});

	theBackend
		.implementOperation('fetch', async (ctx) => {
			// need to genericise the response here so we don't depend on the HTTP responses.

			return new YesResponse({
				statusMessage: 'Yes',
			});
		});

	return {
		operations,
		app: enhancedApp,
		backend: theBackend,
		endpoints: {
			[params.endpointName]: theEndpoint,
		},
	};
};

export class YesResponse extends HttpResponse(204) {}
